/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2019 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: stsn_hts221.c 2416 2019-02-08 17:37:45Z roi $
 */

#include "x_nucleo_iks01a1.h"
#include "stsn_hts221.h"


static SensContext_t TEMPERATURE_SensorHandle[ HTS221_SENSORS_MAX_NUM ];

/*
 *  HT221 I2C쥸ɤ߹
 */
static ER
readreg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToRead, uint8_t *Data )
{
	int i = 0;
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;
	ER ercd;

	/* I2C multi-bytes Read not supported for LPS22HB */
	for(i = 0 ; i < NumByteToRead ; i++){
		/* call I2C_EXPBD Read data bus function */
		if((ercd = psi2c->si2c_read(psi2c, handle->address, (RegAddr + i), &Data[i], 1)) != E_OK){
			return ercd;
		}
	}
	return E_OK;
}

/*
 *  HT221 I2C쥸񤭹
 */
static ER
writereg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToWrite, uint8_t *Data )
{
	int i = 0;
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;
	ER  ercd;

	/* I2C multi-bytes Write not supported for LPS22HB */
	for(i = 0 ; i < NumByteToWrite ; i++){
		/* call I2C_EXPBD Write data bus function */
		if((ercd = psi2c->si2c_write(psi2c, handle->address, (RegAddr + i), &Data[i], 1)) != E_OK){
			return ercd;
		}
	}
	return E_OK;
}

/*
 *  HT221 POWER DOWN⡼
 */
static ER
deActivate(SensContext_t *handle)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~HTS221_PD_MASK;

	return writereg(handle, HTS221_CTRL_REG1, 1, &tmp);
}

/*
 *  HT221 OUTPUT DATA RATE⡼
 */
static ER
set_odr(SensContext_t *handle, HTS221_Odr_et odr)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~HTS221_ODR_MASK;
	tmp |= (uint8_t)odr;

	return writereg(handle, HTS221_CTRL_REG1, 1, &tmp);
}

/*
 *  HT221 WHO_AM_I IDȽ
 */
static ER
HTS221_Check_WhoAmI(SensContext_t *handle )
{
	uint8_t who_am_i = 0x00;
	ER  ercd;

	/* Read WHO AM I register */
	if((ercd = readreg(handle, HTS221_WHO_AM_I_REG, 1, &who_am_i)) != E_OK)
		return ercd;
	if ( who_am_i != handle->who_am_i ){
		return E_OBJ;
	}
	return E_OK;
}


/*
 *  HT221
 *  parameter1  id: HT221 ID
 *  parameter2  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_hts221_init(TEMPERATURE_ID_t id, void **handle )
{
	SENSOR_I2C_Handle_t *psi2c;
	SensContext_t *ctx;
	uint8_t tmp;
	ER  ercd;

	if(id != HTS221_T_0)
		return E_PAR;

	if(TEMPERATURE_SensorHandle[ HTS221_T_0 ].isInitialized == 1){
		*handle = (void *)&TEMPERATURE_SensorHandle[ HTS221_T_0 ];
		/* We have reached the max num of instance for this component */
		return E_OK;
	}

	if((psi2c = Sensor_IO_Init()) == NULL ){
		return E_SYS;
	}

	/* Setup sensor handle. */
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].who_am_i      = HTS221_WHO_AM_I_VAL;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].ifType        = 0; /* I2C interface */
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].address       = HTS221_I2C_ADDRESS;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].instance      = HTS221_T_0;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].isInitialized = 0;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].isEnabled     = 0;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].isCombo       = 1;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].pData         = ( void * )NULL;
	TEMPERATURE_SensorHandle[ HTS221_T_0 ].pSi2c         = psi2c;

	*handle = (void *)&TEMPERATURE_SensorHandle[ HTS221_T_0 ];
	ctx = *handle;

	if((ercd = HTS221_Check_WhoAmI(ctx)) != E_OK)
		return ercd;

	/* Power down the device */
	if((ercd = deActivate((void *)ctx)) != E_OK)
		return ercd;

	/* Enable BDU */
	if((ercd = readreg(ctx, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~HTS221_BDU_MASK;
	tmp |= ((uint8_t)1) << HTS221_BDU_BIT;

	if((ercd = writereg(ctx, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	/* Set default ODR */
	if((ercd = stsn_hts221_set_odr( ctx, ODR_LOW )) != E_OK)
		return ercd;
	ctx->isInitialized = 1;
	return E_OK;
}

/*
 *  HT221λ
 *  parameter1  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_hts221_deinit(SensContext_t **handle )
{
	SensContext_t *ctx;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;
	ctx = *handle;
	if(ctx == NULL)
		return E_OK;

	/* Check if the HTS221 humidity sensor is already initialized. */
	/* If yes, skip the deinitialize function, if not call deinitialize function */
	if(ctx->isInitialized == 1){
		/* Check if the HTS221 humidity sensor is still enabled. */
		/* If yes, skip the disable function, if not call disable function */
		if(ctx->isEnabled == 1){
			/* Power down the device */
			if((ercd = deActivate((void *)ctx)) != E_OK)
				return ercd;
		}
		ctx->isEnabled = 0;
	}
	ctx->isInitialized = 0;
	memset(handle, 0, sizeof(SensContext_t));
	*handle = NULL;
	return E_OK;
}

/*
 *  HT221󥵥֥͡
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_hts221_sensor_enable(SensContext_t *handle)
{
	uint8_t tmp;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already enabled */
	if(handle->isEnabled == 1)
		return E_OK;

	/* Power up the device */
	if((ercd = readreg(handle, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;
	tmp |= HTS221_PD_MASK;
	if((ercd = writereg(handle, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	handle->isEnabled = 1;
	return E_OK;
}

/*
 *  HT221󥵥ǥ֥
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_hts221_sensor_disable(SensContext_t *handle)
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already disabled */
	if(handle->isEnabled == 0)
		return E_OK;

    /* Power down the device */
	if((ercd = deActivate((void *)handle)) != E_OK)
		return ercd;
	handle->isEnabled = 0;
	return E_OK;
}

/*
 *  HT221Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_isinitialized(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isInitialized;
	return E_OK;
}

/*
 *  HT221֥͡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_isenabled(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isEnabled;
	return E_OK;
}

/*
 *  HT221ܷ󥵡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_iscombo(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL || status == NULL)
		return E_PAR;

	*status = handle->isCombo;
	return E_OK;
}

/*
 *  HT221󥹥󥹼
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status 󥹥󥹳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_instance(SensContext_t *handle, uint8_t *instance)
{
	if(handle == NULL || instance == NULL)
		return E_PAR;

	*instance = handle->instance;
	return E_OK;
}

/*
 *  HT221 WHO_AM_I IDؿ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status WHO_AM_I IDǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_whoamI(SensContext_t *handle, uint8_t *who_am_i)
{
	if(handle == NULL || who_am_i == NULL)
		return E_PAR;
	/* Read WHO AM I register */
	return readreg(handle, HTS221_WHO_AM_I_REG, 1, who_am_i);
}

/*
 *  HT221 WHO_AM_I IDȽ
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OK
 */
ER
stsn_hts221_check_whoamI(SensContext_t *handle)
{
	if(handle == NULL)
		return E_PAR;
	return HTS221_Check_WhoAmI(handle);
}

/*
 *  HT221 HUMIDITYͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  humidity HUMIDITYͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_hum(SensContext_t *handle, float *humidity )
{
	int16_t H0_T0_out, H1_T0_out, H_T_out;
	int16_t H0_rh, H1_rh;
	uint8_t buffer[2];
	float   tmp_f;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read data from HTS221. */
	if((ercd = readreg(handle, HTS221_H0_RH_X2, 2, buffer)) != E_OK)
		return ercd;
	H0_rh = buffer[0] >> 1;
	H1_rh = buffer[1] >> 1;

	if((ercd = readreg(handle, HTS221_H0_T0_OUT_L, 2, buffer)) != E_OK)
		return ercd;
	H0_T0_out = (((uint16_t)buffer[1]) << 8) | (uint16_t)buffer[0];

	if((ercd = readreg(handle, HTS221_H1_T0_OUT_L, 2, buffer)) != E_OK)
		return ercd;
	H1_T0_out = (((uint16_t)buffer[1]) << 8) | (uint16_t)buffer[0];

	if((ercd = readreg(handle, HTS221_HR_OUT_L_REG, 2, buffer)) != E_OK)
		return ercd;
	H_T_out = (((uint16_t)buffer[1]) << 8) | (uint16_t)buffer[0];

	tmp_f = (float)(H_T_out - H0_T0_out) * (float)(H1_rh - H0_rh) / (float)(H1_T0_out - H0_T0_out)  +  H0_rh;
	*humidity = ( tmp_f > 100.0f ) ? 100.0
           : ( tmp_f <    0.0f ) ?    0.0
           : tmp_f;
	return E_OK;
}

/*
 *  HT221 TEMPERATUREͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  temperature TEMPERATUREͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_temp(SensContext_t *handle, float *temperature)
{
	int16_t T0_out, T1_out, T_out, T0_degC_x8_u16, T1_degC_x8_u16;
	int16_t T0_degC, T1_degC;
	uint8_t buffer[4], tmp;
	float   tmp_f;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read data from HTS221. */
	if((ercd = readreg(handle, HTS221_T0_DEGC_X8, 2, buffer)) != E_OK)
		return ercd;
	if((ercd = readreg(handle, HTS221_T0_T1_DEGC_H2, 1, &tmp)) != E_OK)
		return ercd;

	T0_degC_x8_u16 = (((uint16_t)(tmp & 0x03)) << 8) | ((uint16_t)buffer[0]);
	T1_degC_x8_u16 = (((uint16_t)(tmp & 0x0C)) << 6) | ((uint16_t)buffer[1]);
	T0_degC = T0_degC_x8_u16 >> 3;
	T1_degC = T1_degC_x8_u16 >> 3;

	if((ercd = readreg(handle, HTS221_T0_OUT_L, 4, buffer)) != E_OK)
		return ercd;

	T0_out = (((uint16_t)buffer[1]) << 8) | (uint16_t)buffer[0];
	T1_out = (((uint16_t)buffer[3]) << 8) | (uint16_t)buffer[2];

	if((ercd = readreg(handle, HTS221_TEMP_OUT_L_REG, 2, buffer)) != E_OK)
		return ercd;

	T_out = (((uint16_t)buffer[1]) << 8) | (uint16_t)buffer[0];
	tmp_f = (float)(T_out - T0_out) * (float)(T1_degC - T0_degC) / (float)(T1_out - T0_out)  +  T0_degC;
	*temperature = ( float )tmp_f;
	return E_OK;
}

/*
 *  HT221 OUTPUT DATA RATEͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATEͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_odr(SensContext_t *handle, float *odr)
{
	HTS221_Odr_et odr_low_level;
	uint8_t tmp;
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, HTS221_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;
	odr_low_level = (HTS221_Odr_et)(tmp & HTS221_ODR_MASK);

	switch(odr_low_level){
	case HTS221_ODR_ONE_SHOT:
		*odr =  0.0f;
		break;
	case HTS221_ODR_1HZ:
		*odr =  1.0f;
		break;
	case HTS221_ODR_7HZ:
		*odr =  7.0f;
		break;
	case HTS221_ODR_12_5HZ:
		*odr = 12.5f;
		break;
	default:
		*odr = -1.0f;
		return E_PAR;
	}
	return E_OK;
}

/*
 *  HT221 OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_hts221_set_odr(SensContext_t *handle, SensorOdr_t odr)
{
	HTS221_Odr_et new_odr;

	if(handle == NULL)
		return E_PAR;

	switch(odr){
	case ODR_LOW:
		new_odr = HTS221_ODR_1HZ;
		break;
	case ODR_MID_LOW:
		new_odr = HTS221_ODR_12_5HZ;
		break;
	case ODR_MID:
		new_odr = HTS221_ODR_12_5HZ;
		break;
	case ODR_MID_HIGH:
		new_odr = HTS221_ODR_12_5HZ;
		break;
	case ODR_HIGH:
		new_odr = HTS221_ODR_12_5HZ;
		break;
	default:
		return E_PAR;
	}
	return set_odr( (void *)handle, new_odr );
}

/*
 *  HT221 OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_hts221_set_odr_value(SensContext_t *handle, float odr)
{
	HTS221_Odr_et new_odr;

	if(handle == NULL)
		return E_PAR;


	new_odr = ( odr <= 1.0f ) ? HTS221_ODR_1HZ
            : ( odr <= 7.0f ) ? HTS221_ODR_7HZ
            :                   HTS221_ODR_12_5HZ;

	return set_odr((void *)handle, new_odr);
}

/*
 *  HT221 󥵡쥸ɤ߽Ф
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   ɤ߽ФǡǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_read_reg(SensContext_t *handle, uint8_t reg, uint8_t *data)
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg( (void *)handle, reg, 1, data)) != E_OK )
		return ercd;
	return E_OK;
}

/*
 *  HT221 󥵡쥸ͽ񤭹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   񤭹ߥǡ
 *  return E_OKｪλ
 */
ER
stsn_hts221_write_reg(SensContext_t *handle, uint8_t reg, uint8_t data )
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = writereg( (void *)handle, reg, 1, &data)) != E_OK)
		return ercd;
	return E_OK;
}

/*
 *  HT221 󥵡DATA READY STATUSɤ߹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ơǼΰ
 *  return E_OKｪλ
 */
ER
stsn_hts221_get_drdy_status(SensContext_t *handle, uint8_t *status)
{
	HTS221_BitStatus_et hum_status_raw;
	HTS221_BitStatus_et temp_status_raw;
	uint8_t tmp;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, HTS221_STATUS_REG, 1, &tmp)) != E_OK)
		return ercd;

	hum_status_raw = (HTS221_BitStatus_et)((tmp & HTS221_HDA_MASK) >> HTS221_H_DA_BIT);
	temp_status_raw = (HTS221_BitStatus_et)(tmp & HTS221_TDA_MASK);

	switch(temp_status_raw){
	case HTS221_SET:
		if(hum_status_raw == HTS221_SET)
			*status = 1;
		else
			*status =0;
		break;
	case HTS221_RESET:
		*status = 0;
		break;
	default:
		return E_SYS;
	}
	return E_OK;
}

