/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2019 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: stsn_lis3mdl.c 2416 2019-02-08 17:38:21Z roi $
 */

#include "x_nucleo_iks01a1.h"
#include "stsn_lis3mdl.h"


static SensContext_t MAGNETO_SensorHandle[ LIS3MDL_SENSORS_MAX_NUM ];


/*
 *  LIS3MDL I2C쥸񤭹
 */
static ER
writereg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToWrite, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	if(NumByteToWrite > 1)
		RegAddr |= 0x80;  /* Enable I2C multi-bytes Write */
	return psi2c->si2c_write(psi2c, handle->address, RegAddr, Data, NumByteToWrite);
}

/*
 *  LIS3MDL I2C쥸ɤ߹
 */
static ER
readreg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToRead, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	if(NumByteToRead > 1)
		RegAddr |= 0x80; /* Enable I2C multi-bytes Read */
	return psi2c->si2c_read(psi2c, handle->address, RegAddr, Data, NumByteToRead);
}

/*
 *  MAG W ƥ४ڥ졼⡼
 */
static ER
MAG_W_SystemOperatingMode(SensContext_t *handle, LIS3MDL_MAG_MD_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LIS3MDL_MAG_CTRL_REG3, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_MD_MASK;
	value |= newValue;

	return writereg(handle, LIS3MDL_MAG_CTRL_REG3, 1, &value);
}

/*
 *  MAG W ե륹
 */
static ER
MAG_W_FullScale(SensContext_t *handle, LIS3MDL_MAG_FS_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LIS3MDL_MAG_CTRL_REG2, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_FS_MASK;
	value |= newValue;

	return writereg(handle, LIS3MDL_MAG_CTRL_REG2, 1, &value);
}

/*
 *  MAG R ե륹ͼФ
 */
static ER
MAG_R_FullScale(SensContext_t *handle, LIS3MDL_MAG_FS_t *value)
{
	ER ercd;

	if((ercd = readreg(handle, LIS3MDL_MAG_CTRL_REG2, 1, (uint8_t *)value)) != E_OK)
		return ercd;
	*value &= LIS3MDL_MAG_FS_MASK; //mask
	return E_OK;
}

/*
 *  MAG W OUTPUT DATA RATE
 */
static ER
MAG_W_OutputDataRate(SensContext_t *handle, LIS3MDL_MAG_DO_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LIS3MDL_MAG_CTRL_REG1, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_DO_MASK;
	value |= newValue;

	return writereg(handle, LIS3MDL_MAG_CTRL_REG1, 1, &value);
}

/*
 *  LIS3MDL󥵡WHO_AM_I IDȽ
 */
static ER
Check_WhoAmI(SensContext_t *handle )
{
	uint8_t who_am_i = 0x00;
	ER  ercd;

	/* Read WHO AM I register */
	if((ercd = readreg(handle, LIS3MDL_MAG_WHO_AM_I_REG, 1, (uint8_t *)&who_am_i)) != E_OK)
		return ercd;

	who_am_i &= LIS3MDL_MAG_WHO_AM_I_BIT_MASK; //coerce
	who_am_i  = who_am_i >> LIS3MDL_MAG_WHO_AM_I_BIT_POSITION; //mask
	if( who_am_i != handle->who_am_i ){
		return E_OBJ;
	}
	return E_OK;
}

/*
 *  LIS3MDL󥵡RAW AXESͼ
 */
static ER
Get_Axes_Raw(SensContext_t *handle, int16_t* pData )
{
	uint8_t regValue[6] = {0, 0, 0, 0, 0, 0};
	uint8_t i, j, k;
	uint8_t numberOfByteForDimension;
	ER  ercd;

	/* Read output registers from LIS3MDL_MAG_OUTX_L to LIS3MDL_MAG_OUTZ_H. */
	numberOfByteForDimension = 6 / 3;

	k = 0;
	for (i = 0; i < 3; i++ ){
		for (j = 0; j < numberOfByteForDimension; j++ ){
			if((ercd = readreg(handle, LIS3MDL_MAG_OUTX_L + k, 1, &regValue[k])) != E_OK)
				return ercd;
			k++;
		}
	}

	/* Format the data. */
	pData[0] = ( ( ( ( int16_t )regValue[1] ) << 8 ) + ( int16_t )regValue[0] );
	pData[1] = ( ( ( ( int16_t )regValue[3] ) << 8 ) + ( int16_t )regValue[2] );
	pData[2] = ( ( ( ( int16_t )regValue[5] ) << 8 ) + ( int16_t )regValue[4] );
	return E_OK;
}


/*
 *  LIS3MDL
 *  parameter1  id: LIS3MDL ID
 *  parameter2  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_init( MAGNETO_ID_t id, void **handle )
{
	SENSOR_I2C_Handle_t *psi2c;
	SensContext_t *ctx;
	uint8_t  value;
	ER  ercd;

	if(id != LIS3MDL_0)
		return E_PAR;

	if(MAGNETO_SensorHandle[ LIS3MDL_0 ].isInitialized == 1){
    /* We have reached the max num of instance for this component */
		return E_PAR;
	}

	if ((psi2c = Sensor_IO_Init()) == NULL ){
		return E_SYS;
	}

	/* Setup sensor handle. */
	MAGNETO_SensorHandle[ LIS3MDL_0 ].who_am_i      = LIS3MDL_MAG_WHO_AM_I;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].ifType        = 0; /* I2C interface */
	MAGNETO_SensorHandle[ LIS3MDL_0 ].address       = LIS3MDL_MAG_I2C_ADDRESS_HIGH;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].instance      = LIS3MDL_0;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].isInitialized = 0;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].isEnabled     = 0;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].isCombo       = 0;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].pData         = ( void * )NULL;
	MAGNETO_SensorHandle[ LIS3MDL_0 ].pSi2c         = ( void *)psi2c;

	*handle = (void *)&MAGNETO_SensorHandle[ LIS3MDL_0 ];
	ctx = *handle;

	if((ercd = Check_WhoAmI(ctx)) != E_OK)
		return ercd;

	/* Operating mode selection - power down */
	if((ercd = MAG_W_SystemOperatingMode( (void *)ctx, LIS3MDL_MAG_MD_POWER_DOWN)) != E_OK)
		return ercd;

	/* Enable BDU */
	if((ercd = readreg(ctx, LIS3MDL_MAG_CTRL_REG5, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_BDU_MASK;
	value |= LIS3MDL_MAG_BDU_ENABLE;

	if((ercd = writereg(ctx, LIS3MDL_MAG_CTRL_REG5, 1, &value)) != E_OK)
		return ercd;

	if((ercd = stsn_lis3mdl_set_odr( ctx, ODR_HIGH )) != E_OK)
		return ercd;

	if((ercd = stsn_lis3mdl_set_fs( ctx, FS_LOW )) != E_OK)
		return ercd;

	/* X and Y axes Operating mode selection */
	if((ercd = readreg(ctx, LIS3MDL_MAG_CTRL_REG1, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_OM_MASK;
	value |= LIS3MDL_MAG_OM_HIGH;

	if((ercd = writereg(ctx, LIS3MDL_MAG_CTRL_REG1, 1, &value)) != E_OK)
		return ercd;

	/* Temperature sensor disable - temp. sensor not used */
	if((ercd = readreg(ctx, LIS3MDL_MAG_CTRL_REG1, 1, &value)) != E_OK)
		return ercd;

	value &= ~LIS3MDL_MAG_TEMP_EN_MASK;
	value |= LIS3MDL_MAG_TEMP_EN_DISABLE;

	if((ercd = writereg(ctx, LIS3MDL_MAG_CTRL_REG1, 1, &value)) != E_OK)
		return ercd;

	ctx->isInitialized = 1;
	return E_OK;
}

/*
 *  LIS3MDLλ
 *  parameter1  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_deinit(SensContext_t **handle )
{
	SensContext_t *ctx = *handle;
	ER  ercd;

	if(ctx == NULL)
		return E_PAR;

	if((ercd = Check_WhoAmI(ctx)) != E_OK)
		return ercd;

	/* Disable the component */
	/* Check if the component is already disabled */
	if(ctx->isEnabled == 0 )
		return E_OK;

	/* Operating mode selection - power down */
	if((ercd = MAG_W_SystemOperatingMode( (void *)handle, LIS3MDL_MAG_MD_POWER_DOWN)) != E_OK)
		return ercd;

	ctx->isEnabled = 0;
	ctx->isInitialized = 0;
	memset(ctx, 0, sizeof(SensContext_t));
	*handle = NULL;
	return E_OK;
}


/*
 *  LIS3MDL󥵥֥͡
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_sensor_enable(SensContext_t *handle )
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already enabled */
	if(handle->isEnabled == 1)
		return E_OK;

	/* Operating mode selection - continuous */
	if((ercd = MAG_W_SystemOperatingMode( (void *)handle, LIS3MDL_MAG_MD_CONTINUOUS)) != E_OK)
		return ercd;

	handle->isEnabled = 1;
	return E_OK;
}

/*
 *  LIS3MDL󥵥ǥ֥
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_sensor_disable(SensContext_t *handle )
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already disabled */
	if(handle->isEnabled == 0 )
		return E_OK;

	/* Operating mode selection - power down */
	if((ercd = MAG_W_SystemOperatingMode( (void *)handle, LIS3MDL_MAG_MD_POWER_DOWN)) != E_OK)
		return ercd;

	handle->isEnabled = 0;
	return E_OK;
}

/*
 *  LIS3MDLȽ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_isinitialized(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isInitialized;
	return E_OK;
}

/*
 *  LIS3MDL֥͡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_isenabled(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isEnabled;
	return E_OK;
}

/*
 *  LIS3MDLܷ󥵡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_iscombo(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isCombo;
	return E_OK;
}

/*
 *  LIS3MDL󥹥󥹼
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status 󥹥󥹳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_instance(SensContext_t *handle, uint8_t *instance )
{
	if(handle == NULL || instance == NULL)
		return E_PAR;
	*instance = handle->instance;
	return E_OK;
}

/*
 *  LIS3MDL WHO_AM_I IDؿ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status WHO_AM_I IDǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_whoamI(SensContext_t *handle, uint8_t *who_am_i )
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read WHO AM I register */
	if((ercd = readreg(handle, LIS3MDL_MAG_WHO_AM_I_REG, 1, (uint8_t *)who_am_i)) != E_OK)
		return ercd;

	*who_am_i &= LIS3MDL_MAG_WHO_AM_I_BIT_MASK; //coerce
	*who_am_i  = *who_am_i >> LIS3MDL_MAG_WHO_AM_I_BIT_POSITION; //mask
	return E_OK;
}

/*
 *  LIS3MDL WHO_AM_I IDȽ
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OK
 */
ER
stsn_lis3mdl_check_whoamI(SensContext_t *handle )
{
	uint8_t who_am_i = 0x00;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = stsn_lis3mdl_get_whoamI(handle, &who_am_i)) != E_OK)
		return ercd;

	if ( who_am_i != handle->who_am_i ){
		return E_OBJ;
	}
	return E_OK;
}

/*
 *  LIS3MDL MAGNETOMETER SENSOR AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  magnetic_field MAGNETOMETER SENSOR AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_axes(SensContext_t *handle, SensorAxes_t *magnetic_field )
{
	int16_t pDataRaw[3];
	float sensitivity = 0;
	LIS3MDL_MAG_FS_t fullScale;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;
	if( magnetic_field == NULL )
		return E_PAR;


	/* Read raw data from LIS3MDL output register. */
	if((ercd = Get_Axes_Raw(handle, pDataRaw)) != E_OK)
		return ercd;

	/* Get LIS3MDL actual sensitivity. */
	if((ercd = MAG_R_FullScale( (void *)handle, &fullScale )) != E_OK)
		return ercd;

	/* Store the sensitivity based on actual full scale. */
	switch( fullScale ){
	case LIS3MDL_MAG_FS_4Ga:
		sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_4G;
		break;
	case LIS3MDL_MAG_FS_8Ga:
		sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_8G;
		break;
	case LIS3MDL_MAG_FS_12Ga:
		sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_12G;
		break;
	case LIS3MDL_MAG_FS_16Ga:
		sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_16G;
		break;
	default:
		sensitivity = -1.0f;
		break;
	}

	/* Calculate the data. */
	magnetic_field->AXIS_X = ( int32_t )( pDataRaw[0] * sensitivity );
	magnetic_field->AXIS_Y = ( int32_t )( pDataRaw[1] * sensitivity );
	magnetic_field->AXIS_Z = ( int32_t )( pDataRaw[2] * sensitivity );
	return E_OK;
}

/*
 *  LIS3MDL MAGNETOMETER SENSOR RAW AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  SensorAxesRaw_t MAGNETOMETER SENSOR RAW AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_axesraw(SensContext_t *handle, SensorAxesRaw_t *value )
{
	int16_t pDataRaw[3];
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read raw data from LIS3MDL output register. */
	if((ercd = Get_Axes_Raw(handle, pDataRaw)) != E_OK)
		return ercd;

	/* Set the raw data. */
	value->AXIS_X = pDataRaw[0];
	value->AXIS_Y = pDataRaw[1];
	value->AXIS_Z = pDataRaw[2];
	return E_OK;
}

/*
 *  LIS3MDL MAGNETOMETER SENSITIVITYͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  sensitivity MAGNETOMETER SENSITIVITYͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_sensitivity(SensContext_t *handle, float *sensitivity)
{
	LIS3MDL_MAG_FS_t fullScale;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read actual full scale selection from sensor. */
	if((ercd = MAG_R_FullScale( (void *)handle, &fullScale)) != E_OK)
		return ercd;

	/* Store the sensitivity based on actual full scale. */
	switch( fullScale ){
	case LIS3MDL_MAG_FS_4Ga:
		*sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_4G;
		break;
	case LIS3MDL_MAG_FS_8Ga:
		*sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_8G;
		break;
	case LIS3MDL_MAG_FS_12Ga:
		*sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_12G;
		break;
	case LIS3MDL_MAG_FS_16Ga:
		*sensitivity = ( float )LIS3MDL_MAG_SENSITIVITY_FOR_FS_16G;
		break;
	default:
		*sensitivity = -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LIS3MDL TEMPERRATUREͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  buff   TEMPERRATUREͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_temperature(SensContext_t *handle, uint8_t *buff)
{
	uint8_t i, j, k;
	uint8_t numberOfByteForDimension;
	ER  ercd;

	numberOfByteForDimension = 2 / 1;

	k = 0;
	for (i = 0; i < 1; i++ ){
		for (j = 0; j < numberOfByteForDimension; j++ ){
			if((ercd = readreg(handle, LIS3MDL_MAG_TEMP_OUT_L + k, 1, &buff[k])) != E_OK)
				return ercd;
			k++;
		}
	}
	return E_OK;
}

/*
 *  LIS3MDL OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_odr(SensContext_t *handle, float *odr )
{
	LIS3MDL_MAG_DO_t odr_low_level;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LIS3MDL_MAG_CTRL_REG1, 1, (uint8_t *)&odr_low_level)) != E_OK)
		return ercd;
	switch((odr_low_level & LIS3MDL_MAG_DO_MASK)){
	case LIS3MDL_MAG_DO_0_625Hz:
		*odr =  0.625f;
		break;
	case LIS3MDL_MAG_DO_1_25Hz:
		*odr =  1.250f;
		break;
	case LIS3MDL_MAG_DO_2_5Hz:
		*odr =  2.500f;
		break;
	case LIS3MDL_MAG_DO_5Hz:
		*odr =  5.000f;
		break;
	case LIS3MDL_MAG_DO_10Hz:
		*odr = 10.000f;
		break;
	case LIS3MDL_MAG_DO_20Hz:
		*odr = 20.000f;
		break;
	case LIS3MDL_MAG_DO_40Hz:
		*odr = 40.000f;
		break;
	case LIS3MDL_MAG_DO_80Hz:
		*odr = 80.000f;
		break;
	default:
		*odr = -1.000f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LIS3MDL OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_set_odr(SensContext_t *handle, SensorOdr_t odr )
{
	LIS3MDL_MAG_DO_t new_odr;

	if(handle == NULL)
		return E_PAR;

	switch( odr ){
	case ODR_LOW:
		new_odr = LIS3MDL_MAG_DO_1_25Hz;
		break;
	case ODR_MID_LOW:
		new_odr = LIS3MDL_MAG_DO_10Hz;
		break;
	case ODR_MID:
		new_odr = LIS3MDL_MAG_DO_40Hz;
		break;
	case ODR_MID_HIGH:
		new_odr = LIS3MDL_MAG_DO_80Hz;
		break;
	case ODR_HIGH:
		new_odr = LIS3MDL_MAG_DO_80Hz;
		break;
	default:
		return E_PAR;
	}

	return MAG_W_OutputDataRate( (void *)handle, new_odr );
}

/*
 *  LIS3MDL OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_set_odr_value(SensContext_t *handle, float odr )
{
	LIS3MDL_MAG_DO_t new_odr;

	if(handle == NULL)
		return E_PAR;

  new_odr = ( odr <=  0.625f ) ? LIS3MDL_MAG_DO_0_625Hz
            : ( odr <=  1.250f ) ? LIS3MDL_MAG_DO_1_25Hz
            : ( odr <=  2.500f ) ? LIS3MDL_MAG_DO_2_5Hz
            : ( odr <=  5.000f ) ? LIS3MDL_MAG_DO_5Hz
            : ( odr <= 10.000f ) ? LIS3MDL_MAG_DO_10Hz
            : ( odr <= 20.000f ) ? LIS3MDL_MAG_DO_20Hz
            : ( odr <= 40.000f ) ? LIS3MDL_MAG_DO_40Hz
            :                     LIS3MDL_MAG_DO_80Hz;

	return MAG_W_OutputDataRate( (void *)handle, new_odr );
}

/*
 *  LIS3MDL 󥵡FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_fs(SensContext_t *handle, float *fullScale )
{
	LIS3MDL_MAG_FS_t fs_low_level;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = MAG_R_FullScale( (void *)handle, &fs_low_level)) != E_OK)
		return ercd;

	switch( fs_low_level ){
	case LIS3MDL_MAG_FS_4Ga:
		*fullScale =  4.0f;
		break;
	case LIS3MDL_MAG_FS_8Ga:
		*fullScale =  8.0f;
		break;
	case LIS3MDL_MAG_FS_12Ga:
		*fullScale = 12.0f;
		break;
	case LIS3MDL_MAG_FS_16Ga:
		*fullScale = 16.0f;
		break;
	default:
		*fullScale = -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LIS3MDL 󥵡FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALE
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_set_fs(SensContext_t *handle, SensorFs_t fullScale)
{
	LIS3MDL_MAG_FS_t new_fs;

	if(handle == NULL)
		return E_PAR;

	switch( fullScale ){
	case FS_LOW:
		new_fs = LIS3MDL_MAG_FS_4Ga;
		break;
	case FS_MID:
		new_fs = LIS3MDL_MAG_FS_8Ga;
		break;
	case FS_HIGH:
		new_fs = LIS3MDL_MAG_FS_12Ga;
		break;
	default:
		return E_PAR;
	}

	return MAG_W_FullScale( (void *)handle, new_fs );
}

/*
 *  LIS3MDL 󥵡FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALE
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_set_fs_value(SensContext_t *handle, float fullScale)
{
	LIS3MDL_MAG_FS_t new_fs;

	if(handle == NULL)
		return E_PAR;

  new_fs = ( fullScale <=  4 ) ? LIS3MDL_MAG_FS_4Ga
           : ( fullScale <=  8 ) ? LIS3MDL_MAG_FS_8Ga
           : ( fullScale <= 12 ) ? LIS3MDL_MAG_FS_12Ga
           :                       LIS3MDL_MAG_FS_16Ga;

	return MAG_W_FullScale( (void *)handle, new_fs );
}

/*
 *  LIS3MDL 󥵡쥸ɤ߽Ф
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   ɤ߽ФǡǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_read_reg(SensContext_t *handle, uint8_t reg, uint8_t *data )
{
	if(handle == NULL)
		return E_PAR;
	return readreg( (void *)handle, reg, 1, data);
}

/*
 *  LIS3MDL 󥵡쥸ͽ񤭹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   񤭹ߥǡ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_write_reg(SensContext_t *handle, uint8_t reg, uint8_t data )
{
	if(handle == NULL)
		return E_PAR;
	return writereg( (void *)handle, reg, 1, &data);
}

/*
 *  LIS3MDL 󥵡DATA READY STATUSɤ߹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ơǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lis3mdl_get_drdy_status(SensContext_t *handle, uint8_t *status )
{
	LIS3MDL_MAG_ZYXDA_t status_raw;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LIS3MDL_MAG_STATUS_REG, 1, (uint8_t *)&status_raw)) != E_OK)
		return ercd;

	switch((status_raw & LIS3MDL_MAG_ZYXDA_MASK)){
	case LIS3MDL_MAG_ZYXDA_AVAILABLE:
		*status = 1;
		break;
	case LIS3MDL_MAG_ZYXDA_NOT_AVAILABLE:
		*status = 0;
		break;
	default:
		return E_SYS;
	}
	return E_OK;
}

