/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2019 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: stsn_lps25hb.c 2416 2019-02-08 17:39:08Z roi $
 */

/*
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2017 STMicroelectronics</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */

#include "x_nucleo_iks01a1.h"
#include "stsn_lps25hb.h"



static SensContext_t PRESSURE_SensorHandle[ LPS25HB_SENSORS_MAX_NUM ];


/*
 *  LPS25HB I2C쥸ɤ߹
 */
static ER
readreg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToRead, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	if(NumByteToRead > 1)
		RegAddr |= 0x80; /* Enable I2C multi-bytes Read */
	/* call I2C_EXPBD Read data bus function */
	return psi2c->si2c_read(psi2c, handle->address, RegAddr, Data, NumByteToRead);
}

/*
 *  LPS25HB I2C쥸񤭹
 */
static ER
writereg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToWrite, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	if(NumByteToWrite > 1)
		RegAddr |= 0x80;  /* Enable I2C multi-bytes Write */
	/* call I2C_EXPBD Write data bus function */
	return psi2c->si2c_write(psi2c, handle->address, RegAddr, Data, NumByteToWrite);
}

/*
 *  LPS25HB WHO_AM_I쥸ɤ߹
 */
static ER
get_deviceID(SensContext_t *handle, uint8_t* deviceid)
{
	return readreg(handle, LPS25HB_WHO_AM_I_REG, 1, deviceid);
}

/*
 *  LPS25HB 󥵡RESOLUTION MODE
 */
static ER
set_avg(SensContext_t *handle, LPS25HB_Avgp_et avgp, LPS25HB_Avgt_et avgt )
{
	ER ercd;
	uint8_t tmp;

	if((ercd = readreg(handle, LPS25HB_RES_CONF_REG, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~(LPS25HB_AVGT_MASK | LPS25HB_AVGP_MASK);
	tmp |= (uint8_t)avgp;
	tmp |= (uint8_t)avgt;
	return writereg(handle, LPS25HB_RES_CONF_REG, 1, &tmp);
}

/*
 *  LPS25HB OUTPUT DATA RATE
 */
static ER
set_Odr(SensContext_t *handle, LPS25HB_Odr_et odr)
{
	uint8_t tmp;
	ER ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~LPS25HB_ODR_MASK;
	tmp |= (uint8_t)odr;
	return writereg(handle, LPS25HB_CTRL_REG1, 1, &tmp);
}

/*
 *  LPS25HB OUTPUT DATA RATE
 */
static ER
get_Odr(SensContext_t *handle, LPS25HB_Odr_et* odr)
{
	uint8_t tmp;
	ER ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;
	*odr = (LPS25HB_Odr_et)(tmp & LPS25HB_ODR_MASK);
	return ercd;
}

/*
 *  LPS25HB POWER DOWMӥå
 */
static ER
set_powerDownMode(SensContext_t *handle, LPS25HB_BitStatus_et pd)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~LPS25HB_PD_MASK;
	tmp |= ((uint8_t)pd) << LPS25HB_PD_BIT;
	return writereg(handle, LPS25HB_CTRL_REG1, 1, &tmp);
}

/*
 *  LPS25HB BLOCK DATA MODE
 */
static ER
set_Bdu(SensContext_t *handle, LPS25HB_Bdu_et bdu)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~LPS25HB_BDU_MASK;
	tmp |= ((uint8_t)bdu);

	return writereg(handle, LPS25HB_CTRL_REG1, 1, &tmp);
}

/*
 *  LPS25HB ̵ͭ
 */
static ER
set_interruptCircuitEnable(SensContext_t *handle, LPS25HB_State_et diff_en)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~LPS25HB_DIFF_EN_MASK;
	tmp |= ((uint8_t)diff_en) << LPS25HB_DIFF_EN_BIT;

	return writereg(handle, LPS25HB_CTRL_REG1, 1, &tmp);
}

/*
 *  LPS25HB SPI󥿡ե
 */
static ER
set_SpiInterface(SensContext_t *handle, LPS25HB_SPIMode_et spimode)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_CTRL_REG1, 1, &tmp)) != E_OK)
		return ercd;

	tmp &= ~LPS25HB_SIM_MASK;
	tmp |= (uint8_t)spimode;

	return writereg(handle, LPS25HB_CTRL_REG1, 1, &tmp);
}

/*
 *  LPS25HB 󥵡ǡơ
 */
static ER
set_DataStatus(SensContext_t *handle, LPS25HB_DataStatus_st* datastatus)
{
	uint8_t tmp;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_STATUS_REG, 1, &tmp)) != E_OK)
		return ercd;

	datastatus->TempDataAvailable = (tmp & LPS25HB_TDA_MASK);
	datastatus->PressDataAvailable = (uint8_t)((tmp & LPS25HB_PDA_MASK) >> LPS25HB_PDA_BIT);
	datastatus->TempDataOverrun = (uint8_t)((tmp & LPS25HB_TOR_MASK) >> LPS25HB_TOR_BIT);
	datastatus->PressDataOverrun = (uint8_t)((tmp & LPS25HB_POR_MASK) >> LPS25HB_POR_BIT);
	return E_OK;
}

/*
 *  LPS25HB RAW PRESSUREͼ
 */
static ER
get_RawPressure(SensContext_t *handle, int32_t *raw_press)
{
	uint8_t buffer[3];
	uint32_t tmp = 0;
	uint8_t i;
	ER  ercd;

	if((ercd = readreg(handle, LPS25HB_PRESS_OUT_XL_REG, 3, buffer)) != E_OK)
		return ercd;

	/* Build the raw data */
	for(i = 0; i < 3; i++){
		tmp |= (((uint32_t)buffer[i]) << (8 * i));
	}

	/* convert the 2's complement 24 bit to 2's complement 32 bit */
	if(tmp & 0x00800000)
		tmp |= 0xFF000000;

	*raw_press = ((int32_t)tmp);
	return E_OK;
}

/*
 *  LPS25HB RAW TEMPERATUREͼ
 */
static ER
get_RawTemperature(SensContext_t *handle, int16_t* raw_data)
{
	uint8_t buffer[2];
	uint16_t tmp;
	ER ercd;

	if((ercd = readreg(handle, LPS25HB_TEMP_OUT_L_REG, 2, buffer)) != E_OK)
		return ercd;

	/* Build the raw tmp */
	tmp = (((uint16_t)buffer[1]) << 8) + (uint16_t)buffer[0];
	*raw_data = ((int16_t)tmp);
	return E_OK;
}

/*
 *  LPS25HB WHO_AM_I IDȽ
 */
static ER
check_WhoamI(SensContext_t *handle)
{
	ER ercd;
	uint8_t who_am_i = 0x00;

	if((ercd = get_deviceID(handle, &who_am_i)) != E_OK)
		return ercd;
	if(who_am_i != handle->who_am_i){
		return E_OBJ;
	}
	return E_OK;
}


/*
 *  LPS25HB
 *  parameter1  id: LPS25HB ID
 *  parameter2  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_init(LPS25HB_ID_t id, void **handle )
{
	SENSOR_I2C_Handle_t *psi2c;
	SensContext_t *ctx;
	ER ercd;

	if(PRESSURE_SensorHandle[ id ].isInitialized == 1)
		return E_PAR;

	if((psi2c = Sensor_IO_Init()) == NULL)
		return E_SYS;

	/* Setup sensor handle. */
	switch(id){
	case LPS25HB_P_0:
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].who_am_i      = LPS25HB_WHO_AM_I_VAL;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].ifType        = 0; /* I2C interface */
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].address       = LPS25HB_ADDRESS_HIGH;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].instance      = LPS25HB_P_0;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].isInitialized = 0;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].isEnabled     = 0;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].isCombo       = 1;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].pData         = ( void * )NULL;
		PRESSURE_SensorHandle[ LPS25HB_P_0 ].pSi2c         = psi2c;
		*handle = (void *)&PRESSURE_SensorHandle[ LPS25HB_P_0 ];
		break;
	case LPS25HB_P_1:
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].who_am_i      = LPS25HB_WHO_AM_I_VAL;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].ifType        = 0; /* I2C interface */
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].address       = LPS25HB_ADDRESS_LOW;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].instance      = LPS25HB_P_1;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].isInitialized = 0;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].isEnabled     = 0;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].isCombo       = 1;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].pData         = ( void * )NULL;
		PRESSURE_SensorHandle[ LPS25HB_P_1 ].pSi2c         = psi2c;
		*handle = (void *)&PRESSURE_SensorHandle[ LPS25HB_P_1 ];
		break;
	default:
		return E_PAR;
	}

	/* Check if the LPS25H/B temperature sensor is already initialized. */
	/* If yes, skip the initialize function, if not call initialize function */
	ctx = (SensContext_t *)*handle;
	if(ctx->isInitialized == 0){

		if((ercd = check_WhoamI(ctx)) != E_OK)
			return ercd;

		/* Power down the device */
		if((ercd = set_powerDownMode(ctx, LPS25HB_RESET)) != E_OK)
			return ercd;

		/* set ODR_LOW */
		dly_tsk(10);
		if((ercd != stsn_lps25hb_set_odr((void *)ctx,  ODR_LOW)) != E_OK)
			return ercd;

		/* Enable interrupt circuit */
		if((ercd = set_interruptCircuitEnable( (void *)ctx, LPS25HB_ENABLE)) != E_OK)
			return ercd;

		/* Set block data update mode */
		if((ercd = set_Bdu( (void *)ctx, LPS25HB_BDU_NO_UPDATE)) != E_OK)
			return ercd;

		/* Set SPI mode */
		if((ercd = set_SpiInterface((void *)ctx, LPS25HB_SPI_3_WIRE)) != E_OK)
			return ercd;

		/* Set internal averaging sample counts for pressure and temperature */
		if((ercd = set_avg((void *)ctx, LPS25HB_AVGP_32, LPS25HB_AVGT_16)) != E_OK)
			return ercd;
	}
	ctx->isInitialized = 1;
	return E_OK;
}

/*
 *  LPS25HBλ
 *  parameter1  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_deinit(SensContext_t **handle)
{
	SensContext_t *ctx = *handle;
	ER  ercd;

	if(ctx == NULL)
		return E_PAR;

	/* Check if the LPS25H/B temperature sensor is already initialized. */
	/* If yes, skip the deinitialize function, if not call deinitialize function */
	if(ctx->isInitialized == 1){
		/* Check if the component is already disabled */
		if(ctx->isEnabled == 0)
			return E_OK;

		if((ercd = set_powerDownMode(ctx, LPS25HB_RESET)) != E_OK)
			return ercd;
		ctx->isEnabled = 0;
	}
	ctx->isInitialized = 0;
	memset(ctx, 0, sizeof(SensContext_t));
	*handle = NULL;
	return E_OK;
}

/*
 *  LPS25HB󥵥֥͡
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_sensor_enable(SensContext_t *handle)
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already enabled */
	if( handle->isEnabled == 1)
		return E_OK;

	/* Check if the LPS25H/B temperature sensor is already enable. */
	/* If yes, skip the enable function, if not call enable function */
	if((ercd = set_powerDownMode(handle, LPS25HB_SET)) != E_OK)
		return ercd;
	handle->isEnabled = 1;
	return COMPONENT_OK;
}

/*
 *  LPS25HB󥵥ǥ֥
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_sensor_disable(SensContext_t *handle)
{
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already disabled */
	if(handle->isEnabled == 0 )
		return E_OK;

	/* Check if the LPS25H/B temperature sensor is still enable. */
	/* If yes, skip the disable function, if not call disable function */
	if((ercd = set_powerDownMode(handle, LPS25HB_RESET)) != E_OK)
		return ercd;
	handle->isEnabled = 0;
	return E_OK;
}

/*
 *  LPS25HBȽ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_isinitialized(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL || status == NULL)
		return E_PAR;
	*status = handle->isInitialized;
	return E_OK;
}

/*
 *  LPS25HB֥͡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_isenabled(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL)
		return E_PAR;
	if(status == NULL)
		return E_PAR;
	*status = handle->isEnabled;
	return E_OK;
}

/*
 *  LPS25HBܷ󥵡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_iscombo(SensContext_t *handle, uint8_t *status)
{
	if(handle == NULL)
		return E_PAR;
	if(status == NULL)
		return E_PAR;
	*status = handle->isCombo;
	return E_OK;
}

/*
 *  LPS25HB󥹥󥹼
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status 󥹥󥹳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_instance(SensContext_t *handle, uint8_t *instance)
{
	if(handle == NULL || instance == NULL)
		return E_PAR;
	*instance = handle->instance;
	return E_OK;
}

/*
 *  LPS25HB WHO_AM_I IDؿ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status WHO_AM_I IDǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_whoamI(SensContext_t *handle, uint8_t *who_am_i)
{
	if(handle == NULL)
		return E_PAR;

	/* Read WHO AM I register */
	return get_deviceID( (void *)handle, who_am_i );
}

/*
 *  LPS25HB WHO_AM_I IDȽ
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OK
 */
ER
stsn_lps25hb_check_whoamI(SensContext_t *handle)
{
	ER ercd;
	uint8_t who_am_i = 0x00;

	if(handle == NULL)
		return E_PAR;

	if((ercd = get_deviceID( (void *)handle, &who_am_i )) != E_OK)
		return ercd;
	if(who_am_i != handle->who_am_i)
		return E_OBJ;
	return ercd;
}

/*
 *  LPS25HB PRESSUREͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  pressure PRESSUREͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_press(SensContext_t *handle, float *pressure)
{
	int32_t raw_press = 0;
	ER  ercd;

	if(handle == NULL || pressure == NULL)
		return E_PAR;
	if((ercd = get_RawPressure(handle, &raw_press)) != E_OK)
		return ercd;
	*pressure = ( float )((raw_press * 100) / 4096) / 100.0f;
	return E_OK;
}

/*
 *  LPS25HB TEMPERATUREͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  temperature TEMPERATUREͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_temp(SensContext_t *handle, float *temperature)
{
	int16_t raw_data = 0;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = get_RawTemperature(handle, &raw_data)) != E_OK)
		return ercd;

	*temperature = ( float )(raw_data / 48 + 425) / 10.0f;
	return E_OK;
}

/*
 *  LPS25HB OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_odr(SensContext_t *handle, float *odr)
{
	LPS25HB_Odr_et odr_low_level;
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = get_Odr((void *)handle, &odr_low_level)) != E_OK)
		return ercd;

	switch(odr_low_level){
	case LPS25HB_ODR_ONE_SHOT:
		*odr =  0.0f;
		break;
	case LPS25HB_ODR_1HZ:
		*odr =  1.0f;
		break;
	case LPS25HB_ODR_7HZ:
		*odr =  7.0f;
		break;
	case LPS25HB_ODR_12_5HZ:
		*odr = 12.5f;
		break;
	case LPS25HB_ODR_25HZ:
		*odr = 25.0f;
		break;
	default:
		*odr = -1.0f;
		return E_PAR;
	}
	return E_OK;
}

/*
 *  LPS25HB OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_set_odr(SensContext_t *handle, SensorOdr_t odr)
{

	LPS25HB_Odr_et new_odr;

	if(handle == NULL)
		return E_PAR;

	switch(odr){
	case ODR_LOW:
		new_odr = LPS25HB_ODR_1HZ;
		break;
	case ODR_MID_LOW:
		new_odr = LPS25HB_ODR_12_5HZ;
		break;
	case ODR_MID:
		new_odr = LPS25HB_ODR_25HZ;
		break;
	case ODR_MID_HIGH:
		new_odr = LPS25HB_ODR_25HZ;
		break;
	case ODR_HIGH:
		new_odr = LPS25HB_ODR_25HZ;
		break;
	default:
		return E_PAR;
	}
	return set_Odr((void *)handle, new_odr);
}

/*
 *  LPS25HB OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_set_odr_value(SensContext_t *handle, float odr)
{

	LPS25HB_Odr_et new_odr;

	if(handle == NULL)
		return E_PAR;

	new_odr = ( odr <=  1.0f ) ? LPS25HB_ODR_1HZ
            : ( odr <=  7.0f ) ? LPS25HB_ODR_7HZ
            : ( odr <= 12.5f ) ? LPS25HB_ODR_12_5HZ
            :                    LPS25HB_ODR_25HZ;

	return set_Odr((void *)handle, new_odr);
}

/*
 *  LPS25HB 󥵡쥸ɤ߽Ф
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   ɤ߽ФǡǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_read_reg(SensContext_t *handle, uint8_t reg, uint8_t *data)
{
	if(handle == NULL)
		return E_PAR;
	return readreg((void *)handle, reg, 1, data );
}

/*
 *  LPS25HB 󥵡쥸ͽ񤭹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   񤭹ߥǡ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_write_reg(SensContext_t *handle, uint8_t reg, uint8_t data)
{
	if(handle == NULL)
		return E_PAR;
	return writereg( (void *)handle, reg, 1, &data );
}

/*
 *  LPS25HB 󥵡DATA READY STATUSɤ߹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ơǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lps25hb_get_drdy_status(SensContext_t *handle, uint8_t *status)
{
	LPS25HB_DataStatus_st status_raw;
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = set_DataStatus((void *)handle, &status_raw)) != E_OK)
		return ercd;

	*status = status_raw.PressDataAvailable;
	return E_OK;
}

