/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2019 by TOPPERS PROJECT Educational Working Group.
 * 
 *  嵭Ԥϡʲ(1)(4)ξ˸¤ꡤܥեȥ
 *  ܥեȥѤΤޤࡥʲƱˤѡʣ
 *  ѡۡʰʲѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *      ޤܥեȥΥ桼ޤϥɥ桼Τʤ
 *      ͳ˴Ťᤫ⡤嵭ԤTOPPERSץȤ
 *      դ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơλŪ
 *  ФŬޤơʤݾڤԤʤޤܥեȥ
 *  ѤˤľŪޤϴŪʤ»˴ؤƤ⡤
 *  Ǥʤ
 * 
 *  $Id: stsn_lsm6ds0.c 2416 2019-02-08 17:39:30Z roi $
 */

/*
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2017 STMicroelectronics</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */

#include "stsn_lsm6ds0.h"



static SensContext_t LSM6DS0_SensorHandle[ LSM6DS0_SENSORS_MAX_NUM ];
static LSM6DS0_Data_t LSM6DS0_Data[ LSM6DS0_SENSORS_MAX_NUM ];

/*
 *  LSM6DS0 I2C쥸񤭹
 */
static ER
writereg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToWrite, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	/* call I2C_EXPBD Write data bus function */
	return psi2c->si2c_write(psi2c, handle->address, RegAddr, Data, NumByteToWrite);
}

/*
 *  LSM6DS0 I2C쥸ɤ߹
 */
static ER
readreg(SensContext_t *handle, uint8_t RegAddr, uint16_t NumByteToRead, uint8_t *Data )
{
	SENSOR_I2C_Handle_t *psi2c = handle->pSi2c;

	/* call I2C_EXPBD Read data bus function */
	return psi2c->si2c_read(psi2c, handle->address, RegAddr, Data, NumByteToRead);
}

/*
 *  LSM6DS0 GYRO FULL SCALE񤭹
 */
static ER
ACC_GYRO_W_GyroFullScale(SensContext_t *handle, LSM6DS0_ACC_GYRO_FS_G_t newValue)
{
	uint8_t value;
	ER ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_FS_G_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, &value);
}

/*
 *  LSM6DS0 GYRO FULL SCALEɤ߹
 */
static ER
ACC_GYRO_R_GyroFullScale(SensContext_t *handle, LSM6DS0_ACC_GYRO_FS_G_t *value)
{
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, (uint8_t *)value)) != E_OK)
		return ercd;
	*value &= LSM6DS0_ACC_GYRO_FS_G_MASK; //mask
	return E_OK;
}

/*
 *  LSM6DS0 GYRO DATA RATE񤭹
 */
static ER
ACC_GYRO_W_GyroDataRate(SensContext_t *handle, LSM6DS0_ACC_GYRO_ODR_G_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_ODR_G_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, &value);
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER FULL SCALE񤭹
 */
static ER
ACC_GYRO_W_AccelerometerFullScale(SensContext_t *handle, LSM6DS0_ACC_GYRO_FS_XL_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_FS_XL_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, &value);
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER FULL SCALEɤ߹
 */
static ER
ACC_GYRO_R_AccelerometerFullScale(SensContext_t *handle, LSM6DS0_ACC_GYRO_FS_XL_t *value)
{
	ER ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, (uint8_t *)value)) != E_OK)
		return ercd;

	*value &= LSM6DS0_ACC_GYRO_FS_XL_MASK; //mask
	return E_OK;
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER DATA RATE񤭹
 */
static ER
ACC_GYRO_W_AccelerometerDataRate(SensContext_t *handle, LSM6DS0_ACC_GYRO_ODR_XL_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_ODR_XL_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, &value);
}

/*
 *  LSM6DS0 GYRO BLOCK DATA UPDATE񤭹
 */
static ER
ACC_GYRO_W_BlockDataUpdate(SensContext_t *handle, LSM6DS0_ACC_GYRO_BDU_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG8, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_BDU_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG8, 1, &value);
}

/*
 *  LSM6DS0 GYRO AXIS X񤭹
 */
static ER
ACC_GYRO_W_GyroAxisX(SensContext_t *handle, LSM6DS0_ACC_GYRO_XEN_G_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_XEN_G_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value);
}

/*
 *  LSM6DS0 GYRO AXIS Y񤭹
 */
static ER
ACC_GYRO_W_GyroAxisY(SensContext_t *handle, LSM6DS0_ACC_GYRO_YEN_G_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_YEN_G_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value);
}

/*
 *  LSM6DS0 GYRO AXIS Z񤭹
 */
static ER
ACC_GYRO_W_GyroAxisZ(SensContext_t *handle, LSM6DS0_ACC_GYRO_ZEN_G_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_ZEN_G_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, &value);
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER AXIS X񤭹
 */
static ER
ACC_GYRO_W_AccelerometerAxisX(SensContext_t *handle, LSM6DS0_ACC_GYRO_XEN_XL_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_XEN_XL_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value);
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER AXIS Y񤭹
 */
static ER
ACC_GYRO_W_AccelerometerAxisY(SensContext_t *handle, LSM6DS0_ACC_GYRO_YEN_XL_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_YEN_XL_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value);
}

/*
 *  LSM6DS0 GYRO ACCELEROMETER AXIS Z񤭹
 */
static ER
ACC_GYRO_W_AccelerometerAxisZ(SensContext_t *handle, LSM6DS0_ACC_GYRO_ZEN_XL_t newValue)
{
	uint8_t value;
	ER  ercd;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_ZEN_XL_MASK;
	value |= newValue;

	return writereg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, &value);
}

/*
 *  LSM6DS0 WHO_AM_I IDȽ
 */
static ER
Check_WhoAmI(SensContext_t *handle)
{
	uint8_t who_am_i = 0x00;
	ER ercd;

	/* Read WHO AM I register */
	if((ercd = stsn_lsm6ds0_get_whoamI(handle, ( uint8_t* )&who_am_i)) != E_OK)
		return ercd;

	if ( who_am_i != handle->who_am_i ){
		return E_OBJ;
	}
	return E_OK;
}

/*
 *  LSM6DS0 DISABLEACCELEROMETER SENSOR OUTPUT DATA RATE
 */
static ER
X_Set_ODR_When_Disabled(SensContext_t *handle, SensorOdr_t odr)
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;

	switch( odr ){
	case ODR_LOW:
		pData->Previous_ODRX = 10.0f;
		break;
	case ODR_MID_LOW:
		pData->Previous_ODRX = 10.0f;
		break;
	case ODR_MID:
		pData->Previous_ODRX = 50.0f;
		break;
	case ODR_MID_HIGH:
		pData->Previous_ODRX = 50.0f;
		break;
	case ODR_HIGH:
		pData->Previous_ODRX = 119.0f;
		break;
	default:
		return E_PAR;
	}
	return E_OK;
}

/*
 *  LSM6DS0 DISABLEGYROSCPE SENSOR OUTPUT DATA RATE
 */
static ER
G_Set_ODR_When_Disabled(SensContext_t *handle, SensorOdr_t odr)
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;

	switch( odr ){
	case ODR_LOW:
		pData->Previous_ODRG = 15.0f;
		pData->lastGyroODR = 15.0f;
		break;
	case ODR_MID_LOW:
		pData->Previous_ODRG = 15.0f;
		pData->lastGyroODR = 15.0f;
		break;
	case ODR_MID:
		pData->Previous_ODRG = 60.0f;
		pData->lastGyroODR = 60.0f;
		break;
	case ODR_MID_HIGH:
		pData->Previous_ODRG = 60.0f;
		pData->lastGyroODR = 60.0f;
		break;
	case ODR_HIGH:
		pData->Previous_ODRG = 119.0f;
		pData->lastGyroODR = 119.0f;
		break;
	default:
		return E_PAR;
	}
	return E_OK;
}

/*
 *  LSM6DS0 ENABLEACCELEROMETER SENSOR OUTPUT DATA RATE
 */
static ER
X_Set_ODR_When_Enabled(SensContext_t *handle, SensorOdr_t odr)
{
	LSM6DS0_ACC_GYRO_ODR_XL_t new_odr;

	switch( odr ){
	case ODR_LOW:
		new_odr = LSM6DS0_ACC_GYRO_ODR_XL_10Hz;
		break;
	case ODR_MID_LOW:
		new_odr = LSM6DS0_ACC_GYRO_ODR_XL_10Hz;
		break;
	case ODR_MID:
		new_odr = LSM6DS0_ACC_GYRO_ODR_XL_50Hz;
		break;
	case ODR_MID_HIGH:
		new_odr = LSM6DS0_ACC_GYRO_ODR_XL_50Hz;
		break;
	case ODR_HIGH:
		new_odr = LSM6DS0_ACC_GYRO_ODR_XL_119Hz;
		break;
	default:
		return E_PAR;
  }

	return ACC_GYRO_W_AccelerometerDataRate(handle, new_odr);
}

/*
 *  LSM6DS0 ENABLEEGYROSCPE SENSOR OUTPUT DATA RATE
 */
static ER
G_Set_ODR_When_Enabled(SensContext_t *handle, SensorOdr_t odr)
{
	LSM6DS0_ACC_GYRO_ODR_G_t new_odr;
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	switch( odr ){
	case ODR_LOW:
		new_odr = LSM6DS0_ACC_GYRO_ODR_G_15Hz;
		break;
	case ODR_MID_LOW:
		new_odr = LSM6DS0_ACC_GYRO_ODR_G_15Hz;
		break;
	case ODR_MID:
		new_odr = LSM6DS0_ACC_GYRO_ODR_G_60Hz;
		break;
	case ODR_MID_HIGH:
		new_odr = LSM6DS0_ACC_GYRO_ODR_G_60Hz;
		break;
	 case ODR_HIGH:
		new_odr = LSM6DS0_ACC_GYRO_ODR_G_119Hz;
		break;
	default:
		return E_PAR;
	}

	if((ercd = ACC_GYRO_W_GyroDataRate(handle, new_odr)) != E_OK)
		return ercd;
	return stsn_lsm6ds0_get_gyro_odr( handle, &pData->lastGyroODR);
}

/*
 *  LSM6DS0 DISABLEACCELEROMETER SENSOR OUTPUT DATA RATE
 */
static ER
X_Set_ODR_Value_When_Disabled(SensContext_t *handle, float odr)
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;

	pData->Previous_ODRX = ( odr <=  10.0f ) ? 10.0f
                                 : ( odr <=  50.0f ) ? 50.0f
                                 : ( odr <= 119.0f ) ? 119.0f
                                 : ( odr <= 238.0f ) ? 238.0f
                                 : ( odr <= 476.0f ) ? 476.0f
                                 :                     952.0f;
	return E_OK;
}

/*
 *  LSM6DS0 DISABLEGYROSCPE SENSOR OUTPUT DATA RATE
 */
static ER
G_Set_ODR_Value_When_Disabled(SensContext_t *handle, float odr)
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;

	pData->Previous_ODRG = ( odr <=  15.0f ) ? 15.0f
                                 : ( odr <=  60.0f ) ? 60.0f
                                 : ( odr <= 119.0f ) ? 119.0f
                                 : ( odr <= 238.0f ) ? 238.0f
                                 : ( odr <= 476.0f ) ? 476.0f
                                 :                     952.0f;

	pData->lastGyroODR = ( odr <=  15.0f ) ? 15.0f
      : ( odr <=  60.0f ) ? 60.0f
      : ( odr <= 119.0f ) ? 119.0f
      : ( odr <= 238.0f ) ? 238.0f
      : ( odr <= 476.0f ) ? 476.0f
      :                     952.0f;
	return E_OK;
}


/*
 *  LSM6DS0 ENABLEACCELEROMETER SENSOR OUTPUT DATA RATE
 */
static ER
X_Set_ODR_Value_When_Enabled(SensContext_t *handle, float odr)
{
	LSM6DS0_ACC_GYRO_ODR_XL_t new_odr;

	new_odr = ( odr <=  10.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_10Hz
            : ( odr <=  50.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_50Hz
            : ( odr <= 119.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_119Hz
            : ( odr <= 238.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_238Hz
            : ( odr <= 476.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_476Hz
            :                     LSM6DS0_ACC_GYRO_ODR_XL_952Hz;

	return ACC_GYRO_W_AccelerometerDataRate(handle, new_odr);
}

/*
 *  LSM6DS0 ENABLEGYROSCPE SENSOR OUTPUT DATA RATE
 */
static ER
G_Set_ODR_Value_When_Enabled(SensContext_t *handle, float odr)
{

	LSM6DS0_ACC_GYRO_ODR_XL_t new_odrx;
	LSM6DS0_ACC_GYRO_ODR_G_t new_odr;
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	new_odrx = ( odr <=  10.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_10Hz
            : ( odr <=  50.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_50Hz
            : ( odr <= 119.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_119Hz
            : ( odr <= 238.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_238Hz
            : ( odr <= 476.0f ) ? LSM6DS0_ACC_GYRO_ODR_XL_476Hz
            :                     LSM6DS0_ACC_GYRO_ODR_XL_952Hz;

	if((ercd = ACC_GYRO_W_AccelerometerDataRate(handle, new_odrx)) != E_OK)
		return ercd;

	new_odr = ( odr <=  15.0f ) ? LSM6DS0_ACC_GYRO_ODR_G_15Hz
            : ( odr <=  60.0f ) ? LSM6DS0_ACC_GYRO_ODR_G_60Hz
            : ( odr <= 119.0f ) ? LSM6DS0_ACC_GYRO_ODR_G_119Hz
            : ( odr <= 238.0f ) ? LSM6DS0_ACC_GYRO_ODR_G_238Hz
            : ( odr <= 476.0f ) ? LSM6DS0_ACC_GYRO_ODR_G_476Hz
            :                     LSM6DS0_ACC_GYRO_ODR_G_952Hz;

	if((ercd = ACC_GYRO_W_GyroDataRate(handle, new_odr)) != E_OK)
		return ercd;

	return stsn_lsm6ds0_get_gyro_odr( handle, &pData->lastGyroODR);
}

/*
 *  LSM6DS0 ACCELEROMETER SENSOR RAW AXES
 */
static ER
X_Get_Axes_Raw(SensContext_t *handle, int16_t *pData)
{
	uint8_t regValue[6] = {0, 0, 0, 0, 0, 0};
	uint8_t i, j, k;
	uint8_t numberOfByteForDimension = 6 / 3;
	ER  ercd;

	k = 0;
	for (i = 0; i < 3; i++ ){
		for (j = 0; j < numberOfByteForDimension; j++ ){
			if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_OUT_X_L_XL + k, 1, &regValue[k])) != E_OK)
				return ercd;
			k++;
		}
	}

	/* Format the data. */
	pData[0] = ( ( ( ( int16_t )regValue[1] ) << 8 ) + ( int16_t )regValue[0] );
	pData[1] = ( ( ( ( int16_t )regValue[3] ) << 8 ) + ( int16_t )regValue[2] );
	pData[2] = ( ( ( ( int16_t )regValue[5] ) << 8 ) + ( int16_t )regValue[4] );
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCPE SENSOR RAW AXES
 */
static ER
G_Get_Axes_Raw(SensContext_t *handle, int16_t *pData)
{
	uint8_t regValue[6] = {0, 0, 0, 0, 0, 0};
	uint8_t i, j, k;
	uint8_t numberOfByteForDimension = 6 / 3;
	ER  ercd;

	/* Read output registers from LSM6DS0_ACC_GYRO_OUT_X_L_G to LSM6DS0_ACC_GYRO_OUT_Z_H_G. */
	k = 0;
	for (i = 0; i < 3; i++ ){
		for (j = 0; j < numberOfByteForDimension; j++ ){
			if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_OUT_X_L_G + k, 1, &regValue[k])) != E_OK)
				return ercd;
			k++;
		}
	}

	/* Format the data. */
	pData[0] = ( ( ( ( int16_t )regValue[1] ) << 8 ) + ( int16_t )regValue[0] );
	pData[1] = ( ( ( ( int16_t )regValue[3] ) << 8 ) + ( int16_t )regValue[2] );
	pData[2] = ( ( ( ( int16_t )regValue[5] ) << 8 ) + ( int16_t )regValue[4] );
	return E_OK;
}


/*
 *  LSM6DS0
 *  parameter1  id: LSM6DS0 ID
 *  parameter2  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_init(LSM6DS0_ID_t id, void **handle)
{
	SENSOR_I2C_Handle_t *psi2c;
	uint8_t axes_status[] = { 1, 1, 1 };
	SensContext_t *ctx;
	uint8_t value;
	ER ercd;

	if(id != LSM6DS0_G_0)
		return E_PAR;

	if(LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].isInitialized == 1){
		*handle = (void *)&LSM6DS0_SensorHandle[ LSM6DS0_G_0 ];
		/* We have reached the max num of instance for this component */
		return E_OK;
	}

	if ((psi2c = Sensor_IO_Init()) == NULL ){
		return E_SYS;
	}

	/* Setup sensor handle. */
	/* Gyroscope - sensor 0 */
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].who_am_i      = LSM6DS0_ACC_GYRO_WHO_AM_I;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].ifType        = 0; /* I2C interface */
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].address       = LSM6DS0_ACC_GYRO_I2C_ADDRESS_HIGH;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].instance      = LSM6DS0_G_0;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].isInitialized = 0;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].isEnabled     = 0;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].isCombo       = 1;
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].pData         = ( void * )&LSM6DS0_Data[ LSM6DS0_G_0 ];
	LSM6DS0_SensorHandle[ LSM6DS0_G_0 ].pSi2c         = ( void * )psi2c;

	*handle = (void *)&LSM6DS0_SensorHandle[ LSM6DS0_G_0 ];
	ctx = *handle;

	if((ercd = Check_WhoAmI(ctx)) != E_OK)
		return ercd;

	/* Enable register address automatically incremented during a multiple byte
	   access with a serial interface. */
	if((ercd = readreg(ctx, LSM6DS0_ACC_GYRO_CTRL_REG8, 1, &value)) != E_OK)
		return ercd;

	value &= ~LSM6DS0_ACC_GYRO_IF_ADD_INC_MASK;
	value |= LSM6DS0_ACC_GYRO_IF_ADD_INC_ENABLE;

	if((ercd = writereg(ctx, LSM6DS0_ACC_GYRO_CTRL_REG8, 1, &value)) != E_OK)
		return ercd;

	/* Select default output data rate. */
	if((ercd = X_Set_ODR_When_Disabled(ctx, ODR_HIGH)) != E_OK)
		return ercd;

	/* Select default output data rate. */
	if((ercd = G_Set_ODR_When_Disabled(ctx, ODR_HIGH)) != E_OK)
		return ercd;

	/* Enable BDU */
	if((ercd = ACC_GYRO_W_BlockDataUpdate(ctx, LSM6DS0_ACC_GYRO_BDU_ENABLE)) != E_OK)
		return ercd;

	/* Output data rate selection - power down. */
	if((ercd = ACC_GYRO_W_GyroDataRate(ctx, LSM6DS0_ACC_GYRO_ODR_G_POWER_DOWN)) != E_OK)
		return ercd;

	/* Full scale selection */
	if((ercd = stsn_lsm6ds0_set_acci_fs( ctx, FS_LOW )) != E_OK)
		return ercd;
	if((ercd = stsn_lsm6ds0_set_gyro_fs( ctx, FS_HIGH)) != E_OK)
		return ercd;

	/* Enable axes */
	if((ercd = stsn_lsm6ds0_set_acci_axes_status(ctx, axes_status)) != E_OK)
		return ercd;
	if((ercd = stsn_lsm6ds0_set_gyro_axes_status(ctx, axes_status)) != E_OK)
		return ercd;
	ctx->isInitialized = 1;
	return COMPONENT_OK;
}

/*
 *  LSM6DS0Lλ
 *  parameter1  handle 󥵥ϥɥǼΰؤΥݥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_deinit(SensContext_t **handle )
{

  SensContext_t *ctx = (SensContext_t *)(*handle);
  LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )ctx->pData;
	ER  ercd;

	if(ctx == NULL)
		return E_OK;

	if((ercd = Check_WhoAmI(ctx)) != E_OK)
		return ercd;

	/* Disable the component */
	if((ercd = stsn_lsm6ds0_gyro_sensor_disable(ctx)) != E_OK)
		return ercd;

	/* Reset output data rate. */
	pData->Previous_ODRX = 0.0f;
	pData->Previous_ODRG = 0.0f;
	ctx->isInitialized = 0;
	memset(ctx, 0, sizeof(SensContext_t));
	*handle = NULL;
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER󥵥֥͡
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_acci_sensor_enable(SensContext_t *handle )
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already enabled */
	if(handle->isEnabled == 1 )
		return E_OK;

	/* Output data rate selection */
	if((ercd = X_Set_ODR_Value_When_Enabled(handle, pData->Previous_ODRX)) != E_OK)
		return ercd;
	handle->isEnabled = 1;
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCOPR󥵥֥͡
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_gyro_sensor_enable(SensContext_t *handle)
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already enabled */
	if(handle->isEnabled == 1)
		return E_OK;

	/* Output data rate selection */
	if((ercd = G_Set_ODR_Value_When_Enabled(handle, pData->Previous_ODRG)) != E_OK)
		return ercd;

	pData->isGyroEnabled = 1;
	handle->isEnabled = 1;
	return E_OK;
}

/*
 *  LIS3MDL ACCELEROMETER󥵥ǥ֥
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_acci_sensor_disable(SensContext_t *handle )
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already disabled */
	if(handle->isEnabled == 0 )
		return E_OK;

	/* Store actual output data rate. */
	if((ercd = stsn_lsm6ds0_get_acci_odr(handle, &( pData->Previous_ODRX))) != E_OK)
		return ercd;

	/* Output data rate selection - power down. */
	if((ercd = ACC_GYRO_W_AccelerometerDataRate(handle, LSM6DS0_ACC_GYRO_ODR_XL_POWER_DOWN)) != E_OK)
		return ercd;

	handle->isEnabled = 0;
	return E_OK;
}

/*
 *  LIS3MDL GYROSCOPE󥵥ǥ֥
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_gyro_sensor_disable(SensContext_t *handle )
{
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Check if the component is already disabled */
	if(handle->isEnabled == 0)
		return E_OK;

	/* Store actual output data rate. */
	if((ercd = stsn_lsm6ds0_get_acci_odr(handle, &(pData->Previous_ODRG))) != E_OK)
		return ercd;

	/* Output data rate selection - power down. */
	if((ercd = ACC_GYRO_W_AccelerometerDataRate(handle, LSM6DS0_ACC_GYRO_ODR_XL_POWER_DOWN)) != E_OK)
		return ercd;

	/* Store actual output data rate. */
	if((ercd = stsn_lsm6ds0_get_gyro_odr(handle, &(pData->Previous_ODRG))) != E_OK)
		return ercd;

	/* Output data rate selection - power down. */
	if((ercd = ACC_GYRO_W_GyroDataRate(handle, LSM6DS0_ACC_GYRO_ODR_G_POWER_DOWN)) != E_OK)
		return ercd;

	pData->isGyroEnabled = 0;
	pData->lastGyroODR = 0.0f;
	handle->isEnabled = 0;
	return E_OK;
}

/*
 *  LSM6DS0Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_isinitialized(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;

	*status = handle->isInitialized;
	return E_OK;
}

/*
 *  LSM6DS0֥͡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_isenabled(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;

	*status = handle->isEnabled;
	return E_OK;
}

/*
 *  LSM6DS0ܷ󥵡Ƚ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ̳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_iscombo(SensContext_t *handle, uint8_t *status )
{
	if(handle == NULL || status == NULL)
		return E_PAR;

	*status = handle->isCombo;
	return E_OK;
}

/*
 *  LSM6DS03MDL󥹥󥹼
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status 󥹥󥹳Ǽΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_instance(SensContext_t *handle, uint8_t *instance )
{
	if(handle == NULL || instance == NULL)
		return E_PAR;

	*instance = handle->instance;
	return E_OK;
}

/*
 *  LSM6DS0 WHO_AM_I IDؿ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status WHO_AM_I IDǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_whoamI(SensContext_t *handle, uint8_t *who_am_i )
{
	if(handle == NULL)
		return E_PAR;
	return readreg(handle, LSM6DS0_ACC_GYRO_WHO_AM_I_REG, 1, (uint8_t *)who_am_i);
}

/*
 *  LSM6DS0 WHO_AM_I IDȽ
 *  parameter1  handle 󥵥ϥɥ
 *  return E_OK
 */
ER
stsn_lsm6ds0_check_whoamI(SensContext_t *handle )
{
	if(handle == NULL)
		return E_PAR;
	return Check_WhoAmI(handle);
}

/*
 *  LSM6DS0 ACCELEROMETER SENSOR AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  acceleration ACCELEROMETER SENSOR AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_acci_axes(SensContext_t *handle, SensorAxes_t *acceleration)
{
	int16_t dataRaw[3];
	float sensitivity = 0;
	ER  ercd;

	/* Read raw data from LSM6DS0 output register. */
	if((ercd = X_Get_Axes_Raw(handle, dataRaw)) != E_OK)
		return ercd;

	/* Get LSM6DS0 actual sensitivity. */
	if((ercd = stsn_lsm6ds0_acci_sensitivity(handle, &sensitivity)) != E_OK)
		return ercd;

	/* Calculate the data. */
	acceleration->AXIS_X = ( int32_t )( dataRaw[0] * sensitivity );
	acceleration->AXIS_Y = ( int32_t )( dataRaw[1] * sensitivity );
	acceleration->AXIS_Z = ( int32_t )( dataRaw[2] * sensitivity );
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE SENSOR AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  angular_velocity GYROSCAPE SENSOR AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_gyro_axes(SensContext_t *handle, SensorAxes_t *angular_velocity)
{
	int16_t dataRaw[3];
	float sensitivity = 0;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read raw data from LSM6DS0 output register. */
	if((ercd = G_Get_Axes_Raw(handle, dataRaw)) != E_OK)
		return ercd;

	/* Get LSM6DS0 actual sensitivity. */
	if((ercd = stsn_lsm6ds0_gyro_sensitivity(handle, &sensitivity)) != E_OK)
		return ercd;

	/* Calculate the data. */
	angular_velocity->AXIS_X = ( int32_t )( dataRaw[0] * sensitivity );
	angular_velocity->AXIS_Y = ( int32_t )( dataRaw[1] * sensitivity );
	angular_velocity->AXIS_Z = ( int32_t )( dataRaw[2] * sensitivity );
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER SENSOR RAW AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  value  ACCELEROMETER SENSOR RAW AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_acci_axesraw(SensContext_t *handle, SensorAxesRaw_t *value)
{
	int16_t dataRaw[3];
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read raw data from LSM6DS0 output register. */
	if((ercd = X_Get_Axes_Raw(handle, dataRaw)) != E_OK)
		return ercd;

	/* Set the raw data. */
	value->AXIS_X = dataRaw[0];
	value->AXIS_Y = dataRaw[1];
	value->AXIS_Z = dataRaw[2];
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE SENSOR RAW AXESͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  value  GYROSCAPE SENSOR RAW AXESͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_gyro_axesraw(SensContext_t *handle, SensorAxesRaw_t *value )
{
	int16_t dataRaw[3];
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read raw data from LSM6DS0 output register. */
	if((ercd = G_Get_Axes_Raw( handle, dataRaw)) != E_OK)
		return ercd;

	/* Set the raw data. */
	value->AXIS_X = dataRaw[0];
	value->AXIS_Y = dataRaw[1];
	value->AXIS_Z = dataRaw[2];
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER SENSITIVITYͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  sensitivity ACCELEROMETER SENSITIVITYͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_acci_sensitivity(SensContext_t *handle, float *sensitivity )
{
	LSM6DS0_ACC_GYRO_FS_XL_t fullScale;
	ER  ercd;

	/* Read actual full scale selection from sensor. */
	if((ercd = ACC_GYRO_R_AccelerometerFullScale(handle, &fullScale)) != E_OK)
		return ercd;

	/* Store the sensitivity based on actual full scale. */
	switch( fullScale ){
	case LSM6DS0_ACC_GYRO_FS_XL_2g:
		*sensitivity = ( float )LSM6DS0_ACC_SENSITIVITY_FOR_FS_2G;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_4g:
		*sensitivity = ( float )LSM6DS0_ACC_SENSITIVITY_FOR_FS_4G;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_8g:
		*sensitivity = ( float )LSM6DS0_ACC_SENSITIVITY_FOR_FS_8G;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_16g:
		*sensitivity = ( float )LSM6DS0_ACC_SENSITIVITY_FOR_FS_16G;
		break;
	default:
		*sensitivity = -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE SENSITIVITYͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  sensitivity GYROSCAPE SENSITIVITYͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_gyro_sensitivity(SensContext_t *handle, float *sensitivity )
{
	LSM6DS0_ACC_GYRO_FS_G_t fullScale;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	/* Read actual full scale selection from sensor. */
	if((ercd = ACC_GYRO_R_GyroFullScale(handle, &fullScale)) != E_OK)
		return ercd;

	/* Store the sensitivity based on actual full scale. */
	switch( fullScale ){
	case LSM6DS0_ACC_GYRO_FS_G_245dps:
		*sensitivity = ( float )LSM6DS0_GYRO_SENSITIVITY_FOR_FS_245DPS;
		break;
	case LSM6DS0_ACC_GYRO_FS_G_500dps:
		*sensitivity = ( float )LSM6DS0_GYRO_SENSITIVITY_FOR_FS_500DPS;
		break;
	case LSM6DS0_ACC_GYRO_FS_G_2000dps:
		*sensitivity = ( float )LSM6DS0_GYRO_SENSITIVITY_FOR_FS_2000DPS;
		break;
	default:
		*sensitivity = -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 TEMPERARUREͼ
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  buff   TEMPERARUREͳǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_temperature(SensContext_t *handle, uint8_t *buff)
{
	uint8_t i, j, k;
	uint8_t numberOfByteForDimension = 2 / 1;
	ER  ercd;

	k = 0;
	for (i = 0; i < 1; i++ ){
		for (j = 0; j < numberOfByteForDimension; j++ ){
			if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_OUT_TEMP_L + k, 1, &buff[k])) != E_OK)
				return ercd;
			k++;
		}
	}
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_acci_odr(SensContext_t *handle, float *odr )
{
	LSM6DS0_ACC_GYRO_ODR_XL_t odr_low_level;
	LSM6DS0_Data_t *pData = ( LSM6DS0_Data_t * )handle->pData;
	ER  ercd;

	/* Accelerometer ODR forced to be same like gyroscope ODR. */
	if(pData->isGyroEnabled == 1){
		*odr = pData->lastGyroODR;
		return E_OK;
	}

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG6_XL, 1, (uint8_t *)&odr_low_level)) != E_OK)
		return ercd;

	switch((odr_low_level & LSM6DS0_ACC_GYRO_ODR_XL_MASK)){
	case LSM6DS0_ACC_GYRO_ODR_XL_POWER_DOWN:
		*odr =   0.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_10Hz:
		*odr =  10.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_50Hz:
		*odr =  50.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_119Hz:
		*odr = 119.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_238Hz:
		*odr = 238.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_476Hz:
		*odr = 476.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_XL_952Hz:
		*odr = 952.0f;
		break;
	default:
		*odr =  -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_gyro_odr(SensContext_t *handle, float *odr )
{
	LSM6DS0_ACC_GYRO_ODR_G_t odr_low_level;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG1_G, 1, (uint8_t *)&odr_low_level)) != E_OK)
	    return ercd;
	odr_low_level &= LSM6DS0_ACC_GYRO_ODR_G_MASK; //mask

	switch( odr_low_level ){
	case LSM6DS0_ACC_GYRO_ODR_G_POWER_DOWN:
		*odr =   0.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_15Hz:
		*odr =  15.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_60Hz:
		*odr =  60.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_119Hz:
		*odr = 119.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_238Hz:
		*odr = 238.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_476Hz:
		*odr = 476.0f;
		break;
	case LSM6DS0_ACC_GYRO_ODR_G_952Hz:
		*odr = 952.0f;
		break;
	default:
		*odr =  -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_acci_odr(SensContext_t *handle, SensorOdr_t odr)
{
	ER ercd;

	if(handle->isEnabled == 1){
		if((ercd = X_Set_ODR_When_Enabled(handle, odr)) != E_OK)
			return ercd;
	}
	else{
		if((ercd = X_Set_ODR_When_Disabled(handle, odr)) != E_OK)
			return ercd;
	}
	return E_OK;
}

/*
 *  LSM6DS0 CYROSCAPE OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_gyro_odr(SensContext_t *handle, SensorOdr_t odr )
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if(handle->isEnabled == 1){
		if((ercd = G_Set_ODR_When_Enabled(handle, odr)) != E_OK)
			return ercd;
	}
	else{
		if((ercd = G_Set_ODR_When_Disabled(handle, odr)) != E_OK)
			return ercd;
	}
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_acci_odr_value(SensContext_t *handle, float odr )
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if(handle->isEnabled == 1){
		if((ercd = X_Set_ODR_Value_When_Enabled(handle, odr)) != E_OK)
			return ercd;
	}
	else{
		if((ercd = X_Set_ODR_Value_When_Disabled(handle, odr)) != E_OK)
			return ercd;
	}
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE OUTPUT DATA RATE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  odr    OUTPUT DATA RATE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_gyro_odr_value(SensContext_t *handle, float odr )
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if(handle->isEnabled == 1){
		if((ercd = G_Set_ODR_Value_When_Enabled(handle, odr)) != E_OK)
			return ercd;
	}
	else{
		if((ercd = G_Set_ODR_Value_When_Disabled(handle, odr)) != E_OK)
			return ercd;
	}
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_acci_fs(SensContext_t *handle, float *fullScale)
{
	LSM6DS0_ACC_GYRO_FS_XL_t fs_low_level;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = ACC_GYRO_R_AccelerometerFullScale(handle, &fs_low_level)) != E_OK)
		return ercd;

	switch( fs_low_level ){
	case LSM6DS0_ACC_GYRO_FS_XL_2g:
		*fullScale =  2.0f;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_4g:
		*fullScale =  4.0f;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_8g:
		*fullScale =  8.0f;
		break;
	case LSM6DS0_ACC_GYRO_FS_XL_16g:
		*fullScale = 16.0f;
		break;
	default:
		*fullScale = -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALEǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_gyro_fs(SensContext_t *handle, float *fullScale )
{
	LSM6DS0_ACC_GYRO_FS_G_t fs_low_level;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = ACC_GYRO_R_GyroFullScale(handle, &fs_low_level)) != E_OK)
		return ercd;

	switch( fs_low_level ){
	case LSM6DS0_ACC_GYRO_FS_G_245dps:
		*fullScale =  245.0f;
		break;
	case LSM6DS0_ACC_GYRO_FS_G_500dps:
		*fullScale =  500.0f;
		break;
	case LSM6DS0_ACC_GYRO_FS_G_2000dps:
		*fullScale = 2000.0f;
		break;
	default:
		*fullScale =   -1.0f;
		return E_SYS;
	}
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_acci_fs(SensContext_t *handle, SensorFs_t fullScale )
{
	LSM6DS0_ACC_GYRO_FS_XL_t new_fs;

	if(handle == NULL)
		return E_PAR;

	switch( fullScale ){
	case FS_LOW:
		new_fs = LSM6DS0_ACC_GYRO_FS_XL_2g;
		break;
	case FS_MID:
		new_fs = LSM6DS0_ACC_GYRO_FS_XL_4g;
		break;
	case FS_HIGH:
		new_fs = LSM6DS0_ACC_GYRO_FS_XL_8g;
		break;
	default:
		return E_PAR;
	}
	return ACC_GYRO_W_AccelerometerFullScale( (void *)handle, new_fs);
}

/*
 *  LSM6DS0 GYROSCAPE FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale SCALE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_gyro_fs(SensContext_t *handle, SensorFs_t fullScale )
{
	LSM6DS0_ACC_GYRO_FS_G_t new_fs;

	if(handle == NULL)
		return E_PAR;

	switch( fullScale ){
	case FS_LOW:
		new_fs = LSM6DS0_ACC_GYRO_FS_G_245dps;
		break;
	case FS_MID:
		new_fs = LSM6DS0_ACC_GYRO_FS_G_500dps;
		break;
	case FS_HIGH:
		new_fs = LSM6DS0_ACC_GYRO_FS_G_2000dps;
		break;
	default:
		return E_PAR;
	}
	return ACC_GYRO_W_GyroFullScale(handle, new_fs);
}

/*
 *  LSM6DS0 ACCELEROMETER FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale FULL SCALE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_acci_fs_value(SensContext_t *handle, float fullScale )
{
	LSM6DS0_ACC_GYRO_FS_XL_t new_fs;

	if(handle == NULL)
		return E_PAR;

	new_fs = ( fullScale <= 2.0f ) ? LSM6DS0_ACC_GYRO_FS_XL_2g
           : ( fullScale <= 4.0f ) ? LSM6DS0_ACC_GYRO_FS_XL_4g
           : ( fullScale <= 8.0f ) ? LSM6DS0_ACC_GYRO_FS_XL_8g
           :                         LSM6DS0_ACC_GYRO_FS_XL_16g;

	return ACC_GYRO_W_AccelerometerFullScale(handle, new_fs);
}

/*
 *  LSM6DS0 GYROSCAPE FULL SCALE
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  fullScale FULL SCALE
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_gyro_fs_value(SensContext_t *handle, float fullScale )
{
	LSM6DS0_ACC_GYRO_FS_G_t new_fs;

	if(handle == NULL)
		return E_PAR;

	new_fs = ( fullScale <= 245.0f ) ? LSM6DS0_ACC_GYRO_FS_G_245dps
           : ( fullScale <= 500.0f ) ? LSM6DS0_ACC_GYRO_FS_G_500dps
           :                           LSM6DS0_ACC_GYRO_FS_G_2000dps;

	return ACC_GYRO_W_GyroFullScale(handle, new_fs);
}

/*
 *  LSM6DS0 ACCELEROMETER AXES STATUS
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  xyz_enabled STATUSǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_acci_axes_status(SensContext_t *handle, uint8_t *xyz_enabled)
{
	LSM6DS0_ACC_GYRO_XEN_XL_t xStatus;
	LSM6DS0_ACC_GYRO_YEN_XL_t yStatus;
	LSM6DS0_ACC_GYRO_ZEN_XL_t zStatus;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, (uint8_t *)&xStatus)) != E_OK)
		return ercd;
	xStatus &= LSM6DS0_ACC_GYRO_XEN_XL_MASK;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, (uint8_t *)&yStatus)) != E_OK)
		return ercd;
	yStatus &= LSM6DS0_ACC_GYRO_YEN_XL_MASK;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG5_XL, 1, (uint8_t *)&zStatus)) != E_OK)
		return ercd;
	zStatus &= LSM6DS0_ACC_GYRO_ZEN_XL_MASK;

	xyz_enabled[0] = ( xStatus == LSM6DS0_ACC_GYRO_XEN_XL_ENABLE ) ? 1 : 0;
	xyz_enabled[1] = ( yStatus == LSM6DS0_ACC_GYRO_YEN_XL_ENABLE ) ? 1 : 0;
	xyz_enabled[2] = ( zStatus == LSM6DS0_ACC_GYRO_ZEN_XL_ENABLE ) ? 1 : 0;
	return E_OK;
}

/*
 *  LSM6DS0 GYOSCAPE AXES STATUS
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  xyz_enabled STATUSǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_gyro_axes_status(SensContext_t *handle, uint8_t *xyz_enabled)
{
	LSM6DS0_ACC_GYRO_XEN_G_t xStatus;
	LSM6DS0_ACC_GYRO_YEN_G_t yStatus;
	LSM6DS0_ACC_GYRO_ZEN_G_t zStatus;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, (uint8_t *)&xStatus)) != E_OK)
		return ercd;
	xStatus &= LSM6DS0_ACC_GYRO_XEN_G_MASK;
	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, (uint8_t *)&yStatus)) != E_OK)
		return ercd;
	yStatus &= LSM6DS0_ACC_GYRO_YEN_G_MASK;
	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_CTRL_REG4, 1, (uint8_t *)&zStatus)) != E_OK)
		return ercd;
	zStatus &= LSM6DS0_ACC_GYRO_ZEN_G_MASK;

	xyz_enabled[0] = ( xStatus == LSM6DS0_ACC_GYRO_XEN_G_ENABLE ) ? 1 : 0;
	xyz_enabled[1] = ( yStatus == LSM6DS0_ACC_GYRO_YEN_G_ENABLE ) ? 1 : 0;
	xyz_enabled[2] = ( zStatus == LSM6DS0_ACC_GYRO_ZEN_G_ENABLE ) ? 1 : 0;
	return E_OK;
}

/*
 *  LSM6DS0 ACCELEROMETER AXES STATUS
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  enable_xyz STATUSǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_acci_axes_status(SensContext_t *handle, uint8_t *enable_xyz)
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = ACC_GYRO_W_AccelerometerAxisX(handle,
			( enable_xyz[0] == 1 ) ? LSM6DS0_ACC_GYRO_XEN_XL_ENABLE : LSM6DS0_ACC_GYRO_XEN_XL_DISABLE )) != E_OK)
		return ercd;

	if((ercd = ACC_GYRO_W_AccelerometerAxisY(handle,
			( enable_xyz[1] == 1 ) ? LSM6DS0_ACC_GYRO_YEN_XL_ENABLE : LSM6DS0_ACC_GYRO_YEN_XL_DISABLE )) != E_OK)
		return ercd;

	if((ercd = ACC_GYRO_W_AccelerometerAxisZ(handle,
			( enable_xyz[2] == 1 ) ? LSM6DS0_ACC_GYRO_ZEN_XL_ENABLE : LSM6DS0_ACC_GYRO_ZEN_XL_DISABLE )) != E_OK)
		return ercd;
	return E_OK;
}

/*
 *  LSM6DS0 GYROSCAPE AXES STATUS
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  enable_xyz STATUSǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_set_gyro_axes_status(SensContext_t *handle, uint8_t *enable_xyz)
{
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = ACC_GYRO_W_GyroAxisX(handle,
			( enable_xyz[0] == 1 ) ? LSM6DS0_ACC_GYRO_XEN_G_ENABLE : LSM6DS0_ACC_GYRO_XEN_G_DISABLE )) != E_OK)
		return ercd;

	if((ercd = ACC_GYRO_W_GyroAxisY(handle,
			( enable_xyz[1] == 1 ) ? LSM6DS0_ACC_GYRO_YEN_G_ENABLE : LSM6DS0_ACC_GYRO_YEN_G_DISABLE )) != E_OK)
		return ercd;

	if((ercd = ACC_GYRO_W_GyroAxisZ(handle,
			( enable_xyz[2] == 1 ) ? LSM6DS0_ACC_GYRO_ZEN_G_ENABLE : LSM6DS0_ACC_GYRO_ZEN_G_DISABLE )) != E_OK)
		return ercd;
	return E_OK;
}

/*
 *  LSM6DS0 󥵡쥸ɤ߽Ф
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   ɤ߽ФǡǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_read_reg(SensContext_t *handle, uint8_t reg, uint8_t *data)
{
	if(handle == NULL)
 		return E_PAR;

	return readreg( (void *)handle, reg, 1, data);
}

/*
 *  LSM6DS0 󥵡쥸ͽ񤭹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  reg    쥸ֹ
 *  parameter3  data   񤭹ߥǡ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_write_reg(SensContext_t *handle, uint8_t reg, uint8_t data)
{
	if(handle == NULL)
		return E_PAR;

	return writereg( (void *)handle, reg, 1, &data);
}

/*
 *  LSM6DS0 󥵡DATA READY STATUSɤ߹
 *  parameter1  handle 󥵥ϥɥ
 *  paramater2  status ơǼΰ
 *  return E_OKｪλ
 */
ER
stsn_lsm6ds0_get_drdy_status(SensContext_t *handle, uint8_t *status )
{
	LSM6DS0_ACC_GYRO_XLDA_t status_rawx;
	LSM6DS0_ACC_GYRO_GDA_t status_raw;
	ER  ercd;

	if(handle == NULL)
		return E_PAR;

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_STATUS_REG, 1, (uint8_t *)&status_rawx)) != E_OK)
		return ercd;

	status_rawx &= LSM6DS0_ACC_GYRO_XLDA_MASK; //mask

	if((ercd = readreg(handle, LSM6DS0_ACC_GYRO_STATUS_REG, 1, (uint8_t *)&status_raw)) != E_OK)
		return ercd;

	status_raw &= LSM6DS0_ACC_GYRO_GDA_MASK; //mask

	switch( status_raw ){
	case LSM6DS0_ACC_GYRO_GDA_UP:
		if(status_rawx == LSM6DS0_ACC_GYRO_XLDA_UP)
			*status = 1;
		else
			*status = 0;
		break;
	case LSM6DS0_ACC_GYRO_GDA_DOWN:
		*status = 0;
		break;
	default:
		return E_SYS;
	}
	return E_OK;
}

